package PlayBot::utils::db::stats;

use strict;
use warnings;
use utf8;

use Moose;
use FindBin;
use List::Util qw( reduce );

use PlayBot::utils::db;

has 'sender'   => ( is => 'ro', isa => 'Str' );
has 'chan'     => ( is => 'ro', isa => 'Str' );
has 'date'     => ( is => 'ro', isa => 'Str' );
has 'count'    => ( is => 'ro', isa => 'Int' );
has 'senders'  => ( is => 'ro', isa => 'HashRef[Str]' );
has 'channels' => ( is => 'ro', isa => 'HashRef[Str]' );
has 'fav'      => ( is => 'ro', isa => 'Int' );

around 'BUILDARGS' => sub {
    my ( $orig, $class, $id ) = @_;

    my $count = 0;
    my $sender;
    my $chan;
    my $date;
    my $senders  = {};
    my $channels = {};

    my $dbh = PlayBot::utils::db::main_session;
    my $sth = $dbh->prepare( '
        select
        date, chan, sender_irc
        from playbot_chan
        where content = ? and chan like \'#%\'
        order by date
    ' );

    $sth->execute($id);
    while ( my $row = $sth->fetch ) {

        # first entry
        if ( not $sender ) {
            $sender = $row->[2];
            $chan   = $row->[1];
            $date   = $row->[0];
        }

        $senders->{ $row->[2] }++;
        $channels->{ $row->[1] }++;
        $count++;
    }

    my $fav;
    $sth = $dbh->prepare( '
        select count(*)
        from playbot_fav
        where id = ?
    ' );
    $sth->execute($id);
    while ( my $row = $sth->fetch ) {
        $fav++;
    }

    $dbh->commit;

    return $class->$orig(
        count    => $count,
        sender   => $sender,
        chan     => $chan,
        date     => $date,
        senders  => $senders,
        channels => $channels,
        fav      => $fav,
    );
};

sub max_sender {
    my $self = shift;
    my $max  = reduce { $self->senders->{$a} > $self->senders->{$b} ? $a : $b }
      keys %{ $self->senders };

    return $max;
}

sub max_channel {
    my $self = shift;
    my $max = reduce { $self->channels->{$a} > $self->channels->{$b} ? $a : $b }
      keys %{ $self->channels };

    return $max;
}

1;
